extern NSString* const sByteArrayIsReadyNotification;

enum eDirection {
	eForwards,
	eBackwards
};

@class ByteSlice, ByteEnumerator;

//deep copying
@interface ByteArray : NSObject <NSCopying> {

}

+ tavlTreeByteArray;
+ fullMemoryByteArray;
+ fullMemoryByteArrayWithData:(NSData*)data;

- (unsigned long long)length;
+ byteArrayWithFile:(NSString*)path;

- (void)copyBytes:(unsigned char*)dst range:(LongRange)lrange;
- (void)insertByteSlice:(ByteSlice*)slice inRange:(LongRange)lrange;
- (void)insertByteArray:(ByteArray*)array inRange:(LongRange)lrange;
- (void)deleteBytesInRange:(LongRange)lrange;

//attempts to return without copying data at location location.  resultLength returns by reference the size of the returned chunk.  If the data must be copied (e.g. it lives on disk), up to bufferSize bytes should be copied into buffer, *resultLength should be set to the copied byte length, and buffer should be returned
- (const unsigned char*)getChunkAtLocation:(unsigned long long)location withLength:(unsigned long*)resultLength withBuffer:(unsigned char*)buffer ofSize:(unsigned)bufferSize;

- (ByteEnumerator*)byteEnumeratorForDirection:(enum eDirection)dir;

- subarrayWithRange:(LongRange)lrange;

//returns ULLONG_MAX if not found
- (unsigned long long)indexOfBytesMatchingBytes:(ByteArray*)findBytes inRange:(LongRange)range direction:(enum eDirection)dir;

//ordered array of byte slices
- (NSArray*)byteSlices;

@end

@interface ByteArray (FileWriting)

- (BOOL)topologicalSortWriteToFile:(NSString*)file rangeMappings:(NSArray**)pMappings mappingsToBreak:(NSArray**)pMappingsToBreak willOverwriteSlices:(BOOL*)isInPlace;

- (void)testTopologicalSortingCorrectness;
- (void)testTopologicalSortingPerformance;

@end
