/*
	Polish.h - Handles copying from a disk image / when running from the user's
		download folder to the folder of the user's choice.  Also handles login items.
		
	Copyright (c)2007 Night Productions, by Darkshadow. All Rights Reserved.
	http://www.nightproductions.net/developer.html
	darkshadow@nightproductions.net
	
	May be used freely, but keep my name/copyright in the header.
	
	I would appreciate any code changes / improvements / localizations made sent to me,
	but I do not make that a condition of using this code.
	
	There is NO warranty of any kind, express or implied; use at your own risk.
	Responsibility for damages (if any) to anyone resulting from the use of this
	code rests entirely with the user.
	
	---------------------------------------------
	 * May 10, 2007 - first version
	 * June 1, 2008
		# Fixed a bug where releasing the shared instance would lead to a crash
		# Added in 10.5's new methods to add / remove login items.
		# Added in ability to read / write the hidden attribute of a login item.
		# Initial support for GC (still needs testing!)
		# Added in ability to read keys from the Info.plist file.
		# General code cleanup.
*/

#import <Cocoa/Cocoa.h>

extern NSString * const DSPolishLoginItemsCheckedKey;
extern NSString * const DSPolishCopyCheckedKey;
extern NSString * const DSPolishVersionCheckedKey;
extern NSString * const DSLoginItemNameKey;
extern NSString * const DSLoginItemPathKey;
extern NSString * const DSLoginItemKindKey;
extern NSString * const DSLoginItemIsHiddenKey;

@interface DSAppServices : NSObject {
	@private
	IBOutlet id delegate;
	NSWindow *dialogWindow;
	NSButton *okButton;
	NSButton *cancelButton;
	NSTextField *messageField;
	NSTextField *copyLocationField;
	NSTextField *copyPathField;
	NSMatrix *copyMatrix;
	NSImageView *iconView;
	NSString *appDisplayName;
	NSString *appBundleName;
	NSString *appVersion;
	NSString *chosenCopyPath;
	NSString *sysAppsFolder;
	NSString *userAppsFolder;
	BOOL dontWriteDefaultsKeys;
	BOOL isCheckingCopy;
	BOOL isCheckingLoginItems;
	BOOL didSetAuthCopySel;
	BOOL failedToLaunchSysEvents;
	BOOL checkForNewVersion;
	SEL authorizedCopySelector;
}

+ (id)sharedInstance;
- (id)initWithDelegate:(id)aDel;

- (void)checkIfShouldCopy;
- (void)checkLoginItems;
- (void)removeDefaultsKeysForNewVersion;

- (BOOL)isRunningOnUnwritableMedia;
- (BOOL)isRunningFromUserDownloadFolder;

- (BOOL)isAlreadyALoginItem;
- (NSArray *)loginItemsList;
- (BOOL)addLoginItemWithPath:(NSString *)aPath DEPRECATED_ATTRIBUTE;
- (BOOL)addLoginItemWithPath:(NSString *)aPath hide:(BOOL)hide;
- (BOOL)removeLoginItemWithPath:(NSString *)aPath;
- (BOOL)addLoginItemWithURL:(NSURL *)anURL DEPRECATED_ATTRIBUTE;
- (BOOL)addLoginItemWithURL:(NSURL *)anURL hide:(BOOL)hide;
- (BOOL)removeLoginItemWithURL:(NSURL *)anURL;
- (BOOL)addToLoginItems;
- (BOOL)removeFromLoginItems;

- (void)didAddToLoginItemsClicked:(id)sender;
- (void)didCancelAddToLoginItemsClicked:(id)sender;

- (void)copyApplication:(id)sender;
- (void)cancelCopy:(id)sender;
- (void)authorizedCopyFinishedSuccessfully:(NSString *)finalCopyPath;
- (void)authorizedCopyFailed:(NSString *)errorMessage;

- (void)setChosenCopyPath:(NSString *)aPath;
- (NSString *)chosenCopyPath;
- (void)setDelegate:(id)sender;
- (id)delegate;
- (void)setAuthorizedCopySelector:(SEL)aSel;
- (SEL)authorizedCopySelector;
- (void)setDontWriteDefaultsKeys:(BOOL)flag;
- (BOOL)dontWriteDefaultsKeys;

- (SEL)addAsLoginItemSelector;
- (SEL)cancelAddAsLoginItemSelector;
- (SEL)copySelector;
- (SEL)cancelCopySelector;
- (NSWindow *)dialogWindow;
- (NSButton *)okButton;
- (NSButton *)cancelButton;
- (NSTextField *)messageField;
- (NSTextField *)copyLocationField;
- (NSTextField *)copyPathField;
- (NSMatrix *)copyMatrix;
- (NSImageView *)iconView;

- (BOOL)copyPath:(NSString *)aPath toPath:(NSString *)destPath error:(NSString **)errorString;
- (BOOL)trashFileAtPath:(NSString *)aPath error:(NSString **)errorString;

@end

@interface NSObject (DSAppServicesDelegateMethods)

- (void)checkForCopyDidFinish;
- (void)checkForAddAsLoginItemDidFinish;
- (BOOL)shouldRemovePossibleOldLoginItemEntry:(NSString *)path;
- (NSWindow *)customizeLoginItemWindow:(NSWindow *)aWin;
- (NSWindow *)customizeCopyWindow:(NSWindow *)aWin;
- (BOOL)copyFailedToLaunchCopyAtPath:(NSString *)path;
- (BOOL)copyShouldTrashOldCopyAtPath:(NSString *)path;
- (void)pathIsDeletableButFailedToBeTrashed:(NSString *)path;
- (void)failedToLaunchSystemEvents;
- (void)loginItemsUpdated:(NSNotification *)notifaction;

@end
